/** 
 *   EnemigoMortal: Un enemigo que podemos matar
 *  
 *   @see Hardware ElemGrafico Juego
 *   @author 1-DAI IES San Vicente 2009/10
 */
 
/* --------------------------------------------------         
   Parte de Death Pit - Remake
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.14  23-Dic-2009  Nacho Cabanes
                      Primera versión del EnemigoMortal
   0.16  01-Feb-2010  Nacho Cabanes
                      EsPosibleMover usa coordenadas inicial y final
                      Añadido velocX y velocY para movimiento más suave
                      Sigue en la misma dirección hasta que choca
   0.17  02-Feb-2010  Estibaliz Garcia
                      Se cambian las imagenes del personaje
   0.19  12-Feb-2010  Nacho Cabanes
                      "Morir" realmente hace algo
                      "Mover" tiene en cuenta si esta muriendo
 ---------------------------------------------------- */

using System; // Para numeros aleatorios: System.Random 
 
public class EnemigoMortal : ElemGrafico
{
    Random numAleatorio;      // Para que se mueva al azar
    protected Partida miPartida;  // Para leer datos del mapa
    short velocX = 3;
    short velocY = 3;
    bool muriendo;     // Para la animacin cuando muere
      
    public EnemigoMortal() {
        numAleatorio = new Random( DateTime.Now.Millisecond );
        ancho = 70;
        alto = 50;
        incrX = velocX;
        incrY = velocY;
        muriendo = false;
    }
    
    
    
    public void Morir() 
    {
        if (muriendo != true)
            muriendo = true;
        
        activo = false;
        CambiarDireccion(MURIENDO);
    }



    public new void Mover() 
    {
        if (muriendo) 
        {
          y += 4;
          SiguienteFotograma();
          if (y > miPartida.GetMapa().GetMaxY() )   // ## TODO: Pedir al mapa
          {
            Recolocar();
            CambiarDireccion(DERECHA);
          }
          return;
        }
        
        // Si puede seguir en la dirección actual, lo hace
        if (miPartida.GetMapa().EsPosibleMover((short) (x+incrX),
            (short) (y+incrY), (short) (x+ancho +incrX),(short) (y+alto +incrY))
            && (x+incrX < miPartida.GetMapa().GetMaxX())
            && (y+incrY < miPartida.GetMapa().GetMaxY()) )
        { 
            x += incrX;
            y += incrY;
            SiguienteFotograma();
            return;
        }
        
        // Si no...
        // Para la siguiente posición, escojo al azar y miro si realmente se puede mover
        bool posibleMover = false;
        do 
        {
            int numeroAzar = (int) numAleatorio.Next(0,100);
            if (numeroAzar < 25) // Intento derecha
            {
                if ((x < miPartida.GetMapa().GetMaxX()) &&  // Si no ha llegado al borde
                    (miPartida.GetMapa().EsPosibleMover((short) (x+velocX),y,
                    (short) (x+ancho +velocX),(short) (y+alto))))
                {
                    incrX = velocX;
                    incrY = 0;
                    CambiarDireccion(DERECHA);
                    posibleMover = true;
                }
            } 
            else if (numeroAzar < 50) // Intento izquierda
            {
                if ((x > 0) &&  // Si no ha llegado al borde
                    (miPartida.GetMapa().EsPosibleMover((short) (x-velocX),y,
                    (short) (x+ancho -velocX),(short) (y+alto))))
                {
                    incrX = (short) -velocX;
                    incrY = 0;
                    CambiarDireccion(IZQUIERDA);
                    posibleMover = true;
                }
            } 
            else if (numeroAzar < 75) // Intento arriba
            {
                if ((y > 0) &&  // Si no ha llegado al borde
                    (miPartida.GetMapa().EsPosibleMover(x, (short) (y-velocY),
                    (short) (x+ancho),(short) (y+alto -velocY))))
                {
                    incrX = 0;
                    incrY = (short) -velocY;
                    posibleMover = true;
                }
            }
            else // Intento abajo
            {
                if ((y < miPartida.GetMapa().GetMaxY()) &&  // Si no ha llegado al borde
                    (miPartida.GetMapa().EsPosibleMover(x, (short) (y+velocY),
                    (short) (x+ancho),(short) (y+alto +velocY))))
                {
                    incrX = 0;
                    incrY = velocY;
                    posibleMover = true;
                }
            }
            
           
           
        } while (! posibleMover);
    }
    
    
    
    // --- Coloca el enemigo en cualquier posición "libre" de la pantalla actual -----
    public void Recolocar()
    {
        do
        {
            x = (short) ((numAleatorio.Next(0,miPartida.GetMapa().GetMaxX()-2)+1)*ancho);
            y = (short) ((numAleatorio.Next(0,miPartida.GetMapa().GetMaxY()-2)+1)*alto);
        } 
        
        // TODO: Falta asignar incrementos "razonables"
        
        while (! miPartida.GetMapa().EsPosibleMover(x,y,(short)(x+ancho),(short)(y+alto)));
        
        visible = true;
        activo = true;
        muriendo = false;
    }
    
} /* fin de la clase EnemigoMortal */


