/** 
 *   Mapa: Mapa de juego
 *  
 *   @see Hardware ElemGrafico Juego
 *   @author 1-DAI IES San Vicente 2009/10
 */
 
/* --------------------------------------------------         
   Parte de Death Pit - Remake
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.14  23-Dic-2009  Nacho Cabanes
                      Primera versin del Mapa, extraida de "Juego.cs"
   0.16  01-Feb-2010  Nacho Cabanes
                      EsPosibleMover usa coordenadas inicial y final
                      Aadidas constantes como CASILLA_ESPACIO
   0.18  08-Feb-2010  Nacho Cabanes
                      El mapa se lee desde fichero (poco robusto aun)
   0.21  20-Feb-2010  Nacho Cabanes
                      El mapa usa "char" en vez de "byte".
                      Cambiada la rutina de lectura, para que varias pantallas
                        adyacentes estn adyacentes tambin en fichero, en
                        vez de una tras otra.
                      Aadido varios nuevos tipos de casillas, entre ellas
                        una nueva pisable: el "suelo exterior superior".
   0.24  28-Ene-2012  Nacho Cabanes
                      Ampliado tamao de las casillas en un 50%
 ---------------------------------------------------- */

using System.IO; // Para leer el mapa desde fichero

public class Mapa
{

    public const char CASILLA_ESPACIO  = '.';
    public const char CASILLA_HORIZ    = '-';
    public const char CASILLA_HORIZAC  = 'v';
    public const char CASILLA_VERT     = '|';
    public const char CASILLA_ESCALERA = 'H';
    public const char CASILLA_SUELOEXTINF = '=';
    public const char CASILLA_SUELOEXTSUP = '_';
    public const char CASILLA_CIELO    = '\'';

    char [,,,] mapa;
    ElemGrafico imagenFondoHoriz,imagenFondoHorizAcido,
      imagenFondoVert, imagenEscalera, imagenCielo,
      imagenSueloSup, imagenSueloInf;
    
    // Tamao de cada pantalla de juego
    int filasPantalla = 7;    
    int columnasPantalla = 7;
    
    // Tamao de cada "casilla" de pantalla (mapa)
    protected const byte ANCHOCASILLA = 105, 
        ALTOCASILLA = 75;

    // Tamao, en "casillas" de cada pantalla 
    protected const byte MAXFILAS = 6;
    protected const byte MAXCOLS = 6;
  
    // Posiciones lmite de movimiento del personaje
    short minX = 0, maxX = 6 * ANCHOCASILLA,
    minY = 0, maxY = 6* ALTOCASILLA;
      
          
     // Situacin de la pantalla actual dentro del mapa de habitaciones
     int filaActual = 0;
     int columnaActual = 0;

     
  
  
  /// Constructor: carga las imagenes del mapa
  public Mapa()
  {
      string linea;
      int maxFilaNiveles, maxColNiveles;
      
      imagenFondoHoriz = new ElemGrafico("imagenes/fondo_horiz.png");
      imagenFondoHorizAcido = new ElemGrafico("imagenes/fondo_acido.png");
      imagenFondoVert = new ElemGrafico("imagenes/fondo_vert.png");
      imagenEscalera = new ElemGrafico("imagenes/fondo_escalera.png");
      imagenCielo = new ElemGrafico("imagenes/fondo_cielo.png");
      imagenSueloSup = new ElemGrafico("imagenes/fondo_suelosup.png");
      imagenSueloInf = new ElemGrafico("imagenes/fondo_sueloinf.png");
      
      // Leo los datos de fichero
      StreamReader fichero;
      fichero = File.OpenText("mapa.dat");
    
      linea = fichero.ReadLine(); // Comentario      
      linea = fichero.ReadLine(); // Cantidad de filas de niveles
      maxFilaNiveles = System.Convert.ToInt32(linea) - 1;
      
      linea = fichero.ReadLine(); // Comentario
      linea = fichero.ReadLine(); // Cantidad de columnas de niveles      
      maxColNiveles = System.Convert.ToInt32(linea) - 1;
      
      linea = fichero.ReadLine(); // Linea en blanco
    
      // Reservamos espacio en memoria para el mapa
      mapa = new char[maxFilaNiveles+1,maxColNiveles+1,
          filasPantalla,columnasPantalla];
          
      // Leo los datos de los niveles
      for (int filaNivelActual = 0; filaNivelActual<=maxFilaNiveles; filaNivelActual++)
      {
          for (int filaActual = 0; filaActual<filasPantalla; filaActual++)
          {
              linea = fichero.ReadLine(); // Linea de datos
              for (int colNivelActual = 0; colNivelActual<=maxColNiveles; colNivelActual++)
              {
                  for (int colActual = 0; colActual<columnasPantalla; colActual++)
                  {
                      mapa[filaNivelActual, colNivelActual,filaActual, colActual] = 
                          linea[colNivelActual*columnasPantalla + colActual];
                  }
              }
          }          
      }
      fichero.Close();    

  }
  

  /// Devuelve el carcter que hay en la posicin [x,y]
  public int GetPosicion(short x, short y)
  {  
    return mapa[filaActual, columnaActual, y, x];
  }

  
  /// Dibuja el mapa en pantalla
  public void Dibujar()
  {
      int fila, colum;
      
      // Dibujo el fondo de la pantalla
      for (fila=0; fila<filasPantalla; fila++)
        for (colum=0; colum<columnasPantalla; colum++) {
      
          //System.Console.Write("Dib {0} {1} {2} {3} -",
          //                 filaActual, columnaActual, fila,colum);
      
          switch (mapa[filaActual, columnaActual, fila,colum]) {
          
            case CASILLA_ESPACIO:  // Espacio en blanco, no dibujo nada
                break;
                
            case CASILLA_HORIZ:
                imagenFondoHoriz.MoverA( (short) (colum*ANCHOCASILLA) , 
                                        (short) (fila*ALTOCASILLA));
                imagenFondoHoriz.DibujarOculta();
                break;
                
            case CASILLA_HORIZAC:
                imagenFondoHorizAcido.MoverA( (short) (colum*ANCHOCASILLA) , 
                                             (short) (fila*ALTOCASILLA));
                imagenFondoHorizAcido.DibujarOculta();
                break;
                
            case CASILLA_VERT:
                imagenFondoVert.MoverA( (short) (colum*ANCHOCASILLA) , 
                                       (short) (fila*ALTOCASILLA));
                imagenFondoVert.DibujarOculta();
                break;
                
            case CASILLA_ESCALERA:
                imagenEscalera.MoverA( (short) (colum*ANCHOCASILLA) , 
                                      (short) (fila*ALTOCASILLA));
                imagenEscalera.DibujarOculta();
                break;
                
            case CASILLA_CIELO:
                imagenCielo.MoverA( (short) (colum*ANCHOCASILLA) , 
                                      (short) (fila*ALTOCASILLA));
                imagenCielo.DibujarOculta();
                break;
                
            case CASILLA_SUELOEXTSUP:
                imagenSueloSup.MoverA( (short) (colum*ANCHOCASILLA) , 
                                      (short) (fila*ALTOCASILLA));
                imagenSueloSup.DibujarOculta();
                break;
                
            case CASILLA_SUELOEXTINF:
                imagenSueloInf.MoverA( (short) (colum*ANCHOCASILLA) , 
                                      (short) (fila*ALTOCASILLA));
                imagenSueloInf.DibujarOculta();
                break;
          }
        }
  }

  public  bool EsPosibleMover(short x, short y, short xMax, short yMax)
  {
      short posX = (short) (x/ANCHOCASILLA);
      short posY = (short) (y/ALTOCASILLA);
      short posXmax = (short) ((xMax-1)/ANCHOCASILLA);
      short posYmax = (short) ((yMax-1)/ALTOCASILLA);
      
      //Para que no se salga por los bordes de la pantalla
      if ((posX < 0) || (posY < 0) || (posX >= MAXCOLS) || (posY >= MAXFILAS))
          return false;

      //Controlar colision con otros bloques
      if ( ((GetPosicion(posX,posY) != CASILLA_ESPACIO) &&
            (GetPosicion(posX,posY) != CASILLA_ESCALERA) &&
            (GetPosicion(posX,posY) != CASILLA_SUELOEXTSUP) )
           ||
           ((GetPosicion(posX,posYmax) != CASILLA_ESPACIO) &&
            (GetPosicion(posX,posYmax) != CASILLA_ESCALERA) &&
            (GetPosicion(posX,posYmax) != CASILLA_SUELOEXTSUP) ) 
           ||
           ((GetPosicion(posXmax,posY) != CASILLA_ESPACIO) &&
            (GetPosicion(posXmax,posY) != CASILLA_ESCALERA) &&
            (GetPosicion(posXmax,posY) != CASILLA_SUELOEXTSUP) ) 
           ||
           ((GetPosicion(posXmax,posYmax) != CASILLA_ESPACIO) &&
            (GetPosicion(posXmax,posYmax) != CASILLA_ESCALERA) &&
            (GetPosicion(posXmax,posYmax) != CASILLA_SUELOEXTSUP) ) 
          )
          
                return false;
      
      // Si no hay obstaculos, se puede mover a esa posicion
      return true;
  }  
  
  
  
  public  void IrAHabitacion(short columnaHab, short filaHab)
  {
      filaActual = filaHab;
      columnaActual = columnaHab;
  }
  
  
  
  public  void IrAHabitacionDerecha()
  {
      if (columnaActual < MAXCOLS)
          columnaActual++;
      
  }
  
  
  public  void IrAHabitacionIzquierda()
  {
      if (columnaActual > 0)
          columnaActual--;
  }
  
  
  public  void IrAHabitacionArriba()
  {
      if (filaActual > 0)
          filaActual--;
  }
  
  
  public  void IrAHabitacionAbajo()
  {
      if (filaActual < MAXFILAS)
          filaActual++;
  }

  
  // Mxima posicin horizontal a la que se puede mover un personaje o enemigo
  public  short GetMaxX()
  {
      return maxX;
  }
  
  
  // Mxima posicin vertical la que se puede mover un personaje o enemigo
  public  short GetMaxY()
  {
      return maxY;
  }
  
} /* fin de la clase Mapa */
