/** 
 *   Partida: Logica de juego para una partida
 *  
 *   @see Hardware
 *   @author 1-DAI IES San Vicente 2009/10
 */

/* --------------------------------------------------         
   Parte de Death Pit - Remake
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  08-Sep-2009  Nacho Cabanes
                      Version inicial: muestra una imagen
   0.02  07-Oct-2009  Nacho Cabanes
                      Mueve el personaje hacia la derecha
                        hasta llegar al margen
   0.03  13-Oct-2009  Nacho Cabanes
                      El personaje se mueve con flechas
                      Un primer enemigo que se mueve a la vez
   0.04  16-Oct-2009  Nacho Cabanes
                      Primera pantalla de presentacion
                      El primer enemigo rebota en los lados
                      Un segundo enemigo nos persigue
   0.05  20-Oct-2009  Nacho Cabanes
                      Array de enemigos (murcielagos)
                       que rebotan en los lados
   0.06  28-Oct-2009  Nacho Cabanes
                      Array bidimensional para imagen de fondo
   0.07  04-Nov-2009  Nacho Cabanes
                      El personaje se mueve solo por los "huecos"
                      Eliminados los enemigos (por ahora)
                      Renombradas variables i,j a fila,colum
   0.08  10-Nov-2009  Nacho Cabanes
                      Varias funciones para que el fuente sea mas modular
   0.09  16-Nov-2009  Nacho Cabanes
                      Varias pantallas conectadas
   0.10  24-Nov-2009  Nacho Cabanes
                      Un enemigo que se mueve "por los huecos"
   0.11  02-Dic-2009  Nacho Cabanes
                      Si chocamos con enemigo perdemos una vida
   0.12  04-Dic-2009  Nacho Cabanes
                      El personaje cambia de imagen segn la direccin
                      Si choco con un enemigo de lado, muere l y me da puntos  
   0.13  11-Dic-2009  Joaqun, Alejandro, Miguel; retoques por Nacho
                      Cuando acaban las vidas o se pulsa ESC, acaba la partida,
                      se vuelve a la presentacin y se puede volver a empezar
   0.14  23-Dic-2009  Nacho Cabanes
                      Primera versin desglosada en clases. Casi todo el 
                      contenido de "Juego" pasa a "Partida".
   0.16  29-Ene-2010  Nacho Cabanes
                      ComprobarColisiones usa Colision, en vez de comparar
                      coordenadas.
   0.20  15-Feb-2010  Nacho Cabanes
                      Cambia el tipo de enemigo al azar al recolocar
   0.21  20-Feb-2010  Nacho Cabanes
                      Nueva posicin de inicio, en "inicializarPartida"
   0.22  22-Feb-2010  Nacho Cabanes
                      Animacin al morir el personaje
                        "MoverElementos" llama a personaje.Mover(),
                        aadidos "Terminar()" y "GetEnemigo()"
   0.23  25-May-2010  Gema vicedo , Jose Carlos Rabasco
                      El personaje se mueve tambin con ratn
   0.24  28-Ene-2012  Nacho Cabanes
                      Ampliado tamao de personaje, enemigo 
                        y casillas de mapa en un 50%
 ---------------------------------------------------- */

using System; // Para Random

 
public class Partida
{

    // Componentes del juego
    private Mapa miMapa;
    private EnemigoMortal enemigo;
    private Personaje personaje;
    
    ElemGrafico imagenPersonaje;  // Imagen del personaje (se muestra al perder vidas)
    Fuente fuente18;   // Tipo de letra para mensajes
    int puntos = 0;
        
    
    bool partidaTerminada;  // Si ha terminado una partida puntual
    

    // Inicializacin al comenzar la sesin de juego
    public Partida()
    {
        // Inicializo componentes del juego
        miMapa = new Mapa();
        enemigo = new Murcielago(this);
        personaje = new Personaje(this);
        
        
        // Cargo imagenes y tipos de letra
        imagenPersonaje = new ElemGrafico("imagenes/personaje.png");        
        fuente18 = new Fuente("FreeSansBold.ttf", 18);
    }
    
    
    
    // --- Comienzo de un nueva partida: reiniciar variables ---
     void inicializarPartida()
    {    
        puntos = 0;
        personaje.MoverA(105,75);
        personaje.SetVidas(5);
        miMapa.IrAHabitacion(4,1);        
        enemigo.Recolocar();

        partidaTerminada = false;
        MostrarVidasRestantes();
    }
    
  
    
    // --- Comprobar teclas para mover personaje o terminar
     void comprobarTeclas()
    {
          // Muevo si se pulsa alguna flecha del teclado
          if (Hardware.TeclaPulsada(Hardware.TECLA_DER) )
              personaje.MoverDerecha();
            
          if(Hardware.TeclaPulsada(Hardware.TECLA_ARR) )
              personaje.MoverArriba();
          
          if (Hardware.TeclaPulsada(Hardware.TECLA_IZQ) )
              personaje.MoverIzquierda();         
     
          if (Hardware.TeclaPulsada(Hardware.TECLA_ABA) )
              personaje.MoverAbajo();                  
              
          if (Hardware.TeclaPulsada(Hardware.TECLA_ESC))
              partidaTerminada = true;

          // Compruebo el raton
          int posXRaton, posYRaton;
          if (Hardware.RatonPulsado(out posXRaton, out posYRaton))
          {
              int distanciaX = Math.Abs( posXRaton - personaje.GetX() );
              int distanciaY = Math.Abs(posYRaton - personaje.GetY());

              if (distanciaX > distanciaY)  // Movimiento horizontal
              {
                  if (posXRaton > personaje.GetX()) personaje.MoverDerecha();
                  if (posXRaton < personaje.GetX()) personaje.MoverIzquierda();
              }
              else  // Movimiento vertical
              {
                  if (posYRaton > personaje.GetY()) personaje.MoverAbajo();
                  if (posYRaton < personaje.GetY()) personaje.MoverArriba();
              }
          }
    }


    
    // --- Animacin de los enemigos y dems objetos "que se muevan solos" -----
     void moverElementos()
    {
        enemigo.Mover();
        personaje.Mover();
    }
    
    
    
    // --- Comprobar colisiones de enemigo con personaje, etc ---
     void comprobarColisiones()
    {
        if (personaje.ColisionCon(enemigo))
        {
             if ((personaje.GetDireccion() == ElemGrafico.DERECHA) 
                 || (personaje.GetDireccion() == ElemGrafico.IZQUIERDA))
            matarEnemigo();
          else
            morir();
        }
    }
    
    
    
    // --- Matar enemigo: aumentar puntos y recolocarlo ---
    void matarEnemigo()
    {
        puntos += 10;
        enemigo.Morir();
    }
    
    
    
    // --- Recolocar enemigo (llamado al cambiar de habit. p.ej.) ---
    public void RecolocarEnemigo()
    {
        
        // Elijo un nmero del 0 al 2
        Random numAleatorio = new Random( DateTime.Now.Millisecond );
        int numeroAzar = (int) numAleatorio.Next(0,3);
        
        // Cambio el tipo de enemigo al azar
        if (numeroAzar == 0)
            enemigo = new Murcielago(this);
        else if (numeroAzar == 1)
            enemigo = new Escorpion(this);
        else
            enemigo = new Aranya(this);
            
        // Y recoloco
        enemigo.Recolocar();
    }
     

    
    // --- Morir personaje: mostrar vidas restantes, etc ---
    void morir()
    {
        personaje.Morir();
    }
    
    
    
    // --- Mostrar vidas restantes ---
    public void MostrarVidasRestantes()
    {
        int i;
        
        // Rectngulo verde
        Hardware.RectanguloRellenoRGBA(
           0,0,735,525,  // Posicion, ancho y alto de la pantalla
           0,255,0,      // En color verde
           200);         // Con algo de transparencia
        
        
        // Dibujo las vidas restantes
        Hardware.EscribirTextoOculta(
                "Te quedan",
                280, 100, 0, 0, 0, fuente18);
        Hardware.EscribirTextoOculta(
                personaje.GetVidas().ToString(),
                280, 150, 0xFF, 0, 0, fuente18);
        Hardware.EscribirTextoOculta(
                "vidas",
                270, 200, 0, 0, 0, fuente18);
                
        for (i=0; i<personaje.GetVidas(); i++)
        {
            imagenPersonaje.MoverA( 130 , (short) (50+ i*75) );
            imagenPersonaje.DibujarOculta();
        }

        // Y quito el aviso de "ESC para salir"
        Hardware.RectanguloRellenoRGBA(
                160, 550, 800, 580,
                0, 0, 0,
                255);

        // Aviso de la tecla para volver
        Hardware.EscribirTextoOculta(
                "Pulsa V para volver al juego",
                120, 550, 0xAA, 0xAA, 0xAA, fuente18);                
        
        
        // Finalmente, muestro en pantalla
        Hardware.VisualizarOculta();  
        
        // Espero hasta que se pulse la tecla de volver
        do {
        
        } while (! Hardware.TeclaPulsada(Hardware.TECLA_V) );        
    }
    
    
    
    // --- Dibujar en pantalla todos los elementos visibles del juego ---
     void dibujarElementos()
    {
          Hardware.BorrarPantallaOculta(0,0,0);
          
          miMapa.Dibujar();
          personaje.DibujarOculta();
          enemigo.DibujarOculta();
                    
          Hardware.EscribirTextoOculta(
                "Puntos",
                550, 550, 0xAA, 0xAA, 0xAA, fuente18);
          Hardware.EscribirTextoOculta(
                puntos.ToString(),
                700, 550, 0xAA, 0xAA, 0xAA, fuente18);
          
          Hardware.EscribirTextoOculta(
                "Pulsa ESC para salir",
                170, 550, 0xAA, 0xAA, 0xAA, fuente18);
        
          // Finalmente, muestro en pantalla
          Hardware.VisualizarOculta();        
    }
    
    
    
    // --- Pausa tras cada fotograma de juego, para velocidad de 25 fps -----
     void pausaFotograma() 
    {
        Hardware.Pausa( 40 );
    }
    
    
    
    // --- Bucle principal de juego -----
    public void BuclePrincipal() 
    {
        inicializarPartida();
        do {
            comprobarTeclas();
            moverElementos();
            comprobarColisiones();
            dibujarElementos();
            pausaFotograma();
        } while (! partidaTerminada);
    }
    
    
    
    // --- Devuelve el mapa, para que lo consulten el personaje
    // y el enemigo antes de moverse -----
    public Mapa GetMapa() 
    {
      return miMapa;
    }
    
    // --- Devuelve el enemigo actual, para que el personaje pueda
    // saber cosas de el -----
    public EnemigoMortal GetEnemigo() 
    {
      return enemigo;
    }
    
    // --- Devuelve el enemigo actual, para que el personaje pueda
    // saber cosas de el -----
    public void Terminar() 
    {
      partidaTerminada = true;
    }

    
} /* fin de la clase Juego */

