/** 
 *   Personaje: El personaje que es controlado por el usuario
 *  
 *   @see Hardware ElemGrafico Juego
 *   @author 1-DAI IES San Vicente 2009/10
 */

/* --------------------------------------------------         
   Parte de Death Pit - Remake
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.14  23-Dic-2009  Nacho Cabanes
                      Primera versin del Personaje, desglosado de "Juego.cs"
                      No aade posibilidades nuevas.
   0.16  01-Feb-2010  Nacho Cabanes
                      EsPosibleMover usa coordenadas inicial y final
                      Movimiento ms suave: 5 pixeles en cada salto   
   0.17  02-Feb-2010  Estibaliz Garcia
                      Se cambian las imagenes del personaje
   0.22  22-Feb-2010  Nacho Cabanes
                      Animacin al morir: aadidos "muriendo" y "Mover()",
                        ampliado "Morir()"
   0.23  25-May-2010  Gemma Vicedo, Jose Carlos Rabasco
                      Corregido el cambio de direccion (arriba/abajo)
                        para su correcta utilizacion con el raton.
   0.24  28-Ene-2012  Nacho Cabanes
                      Ampliado tamao en un 50%
 ---------------------------------------------------- */

public class Personaje : ElemGrafico
{

  // Datos del personaje
  short vidas;
  short yAntesAnimacion; // Para volver a la psoicion tras la animacion de muerte
  bool muriendo;
  protected Partida miPartida;
    
  public Personaje(Partida p) 
  {
    miPartida = p;
    vidas = 5;
    x = 1;
    y = 1;
    yAntesAnimacion = 0;
    incrX = 5;
    incrY = 5;
    //Antes era: CargarImagen("imagenes/personaje.png");
    direccion = IZQUIERDA;
    CargarSecuencia( DERECHA, 
      new string[] {"imagenes/personajed1.png","imagenes/personajed1.png","imagenes/personajed1.png",
                    "imagenes/personajed2.png","imagenes/personajed2.png","imagenes/personajed2.png"} );
    CargarSecuencia( IZQUIERDA, 
      new string[] {"imagenes/personajei1.png","imagenes/personajei1.png","imagenes/personajei1.png",
                    "imagenes/personajei2.png","imagenes/personajei2.png","imagenes/personajei2.png"} );
    CargarSecuencia( ARRIBA, 
      new string[] {"imagenes/personajea1.png","imagenes/personajea1.png","imagenes/personajea1.png",
                    "imagenes/personajea2.png","imagenes/personajea2.png","imagenes/personajea2.png"} );
    CargarSecuencia( ABAJO, 
      new string[] {"imagenes/personajea1.png","imagenes/personajea1.png","imagenes/personajea1.png",
                    "imagenes/personajea2.png","imagenes/personajea2.png","imagenes/personajea2.png"} );
    CargarSecuencia( MURIENDO, 
      new string[] {"imagenes/personajem1.png","imagenes/personajem1.png","imagenes/personajem1.png",
                    "imagenes/personajem2.png","imagenes/personajem2.png","imagenes/personajem2.png",
                    "imagenes/personajem3.png","imagenes/personajem3.png","imagenes/personajem3.png"} );
    muriendo = false;
    ancho =105;
    alto = 75;
  }
  
  
  
  public void MoverDerecha() 
  {
      if (muriendo) return;
      
      CambiarDireccion(DERECHA);
      if (x >= miPartida.GetMapa().GetMaxX()-5)
      {
        x = 0;
        miPartida.GetMapa().IrAHabitacionDerecha();
        miPartida.RecolocarEnemigo();
      }
      else if (x < miPartida.GetMapa().GetMaxX())
        if (miPartida.GetMapa().EsPosibleMover((short) (x+incrX),y,
          (short) (x+ancho +incrX),(short) (y+alto)))
      {
          x += incrX;
          SiguienteFotograma();
      }
  }
  
  
  
  public void MoverIzquierda() 
  {
      if (muriendo) return;
      
      CambiarDireccion(IZQUIERDA);
      if (x <= 5)
      {
        x = miPartida.GetMapa().GetMaxX();
        miPartida.GetMapa().IrAHabitacionIzquierda();
        miPartida.RecolocarEnemigo();
      }
      else if ( x > 0)            
        if (miPartida.GetMapa().EsPosibleMover((short) (x-incrX),y,
              (short) (x+ancho -incrX),(short) (y+alto)))
      {
            x -= incrX;
            SiguienteFotograma();
      }
  }
  
  
  
  public void MoverArriba()
  {
      if (muriendo) return;
            
      if (y <= 5)
      {
          CambiarDireccion(ARRIBA);
          y = miPartida.GetMapa().GetMaxY();
          miPartida.GetMapa().IrAHabitacionArriba();
          miPartida.RecolocarEnemigo();
      }
      else if ( y > 0)
        if (miPartida.GetMapa().EsPosibleMover(x, (short) (y-incrY),
                (short) (x+ancho),(short) (y+alto -incrY)))
      {
            CambiarDireccion(ARRIBA);
            y -= incrY;
            SiguienteFotograma();
      }
  }
  
  
  
  public void MoverAbajo() 
  {
      if (muriendo) return;
      
      
      if (y >= miPartida.GetMapa().GetMaxY()-5)
      {
          CambiarDireccion(ABAJO);
          y = 0;
          miPartida.GetMapa().IrAHabitacionAbajo();
          miPartida.RecolocarEnemigo();
      }
      else 
      if (y < miPartida.GetMapa().GetMaxY())
        if (miPartida.GetMapa().EsPosibleMover(x, (short) (y+incrY),
                (short) (x+ancho),(short) (y+alto +incrX)))
      {
            CambiarDireccion(ABAJO);
            y += incrY;
            SiguienteFotograma();
      }
  }

  
  
  // Animacin cuando muere  
  public new void Mover()
  {
    if (! muriendo) return;
   
    y += 4;
    SiguienteFotograma();
    if (y > miPartida.GetMapa().GetMaxY() )
    {
        CambiarDireccion(DERECHA);
        muriendo = false;
        y = yAntesAnimacion;
        
        miPartida.MostrarVidasRestantes();
        
        // Vuelvo a colocar el enemigo al azar
        miPartida.GetEnemigo().Recolocar();
        
        // Si no hay vidas, se acab la partida
        if (vidas == 0)
            miPartida.Terminar();        
    }  
  }
  
  
  
  public int GetVidas()
  {
    return vidas;
  }
  
  
  public void SetVidas(short n)
  {
    vidas = n;
  }

  
  public void Morir()
  {
      // Si hay colisin, pero ya est muerto, salgo
      if (muriendo) return;
      
      // Pero si acaba de entrar, empieza la animacion
      yAntesAnimacion = y;
      vidas--;
      CambiarDireccion(MURIENDO);
      muriendo = true;
  }
  

} /* fin de la clase Personaje */
